from lib.locale import translation

translation["ru"] = {
    # Эмоции
    "Anger " : "Злость",
    "Contm." : "Презр.",
    "Disgu." : "Отврщ.",
    "Fear  " : "Страх ",
    "Happs." : "Радст.",
    "Neutr." : "Нейтр.",
    "Sadns." : "Грусть",
    "Surpr." : "Удивл.",

    # Всё остальное в алфавитном порядке
    "Analyze all faces" : "Анализировать все лица",
    "Beep on warning (depend on system settings)" : "Звуковое предупреждение (зависит от системных настроек)",
    "Can't open camera {}" : "Не удаётся открыть камеру {}",
    "Can't read camera image" : "Не удаётся получить изображение с камеры",
    "Camera number" : "Номер камеры",
    "Color Chooser" : "Выбор цвета",
    "Common settings" : "Общие настройки",
    "Config saved" : "Настройки сохранены",
    "Error" : "Ошибка",
    "FPS" : "Кадры в секунду",
    "Limits settings" : "Настройки пределов",
    "Lower limits" : "Нижние границы",
    "Reset" : "Сбросить",
    "Save" : "Сохранить",
    "Scanning directory {} for statistics" : "Открываю статистику в каталоге {}",
    "Show camera picture" : "Показывать изображение с камеры",
    "Show warning" : "Показывать предупреждение",
    "Trying to read the config file " : "Пробую прочитать файл настроек ",
    "Video height" : "Высота видео",
    "Video width" : "Ширина видео",
    "Warning color" : "Цвет предупреждения",
    "Warning delay in seconds" : "Задержка предупреждения в секундах",
    "Warning window size" : "Размер окна с предупреждением",
    "Warning window X pos" : "Координата X предупреждения",
    "Warning window Y pos" : "Координата Y предупреждения",
    "Write stat file" : "Сохранять статистику в файл",
    "Writing to the config file " : "Записываю файл настроек ",
    "Upper limits" : "Верхние границы",
}

# vi: tabstop=4 shiftwidth=4 expandtab
